/****************************************************************************

 KHotKeys
 
 Copyright (C) 1999-2001 Lubos Lunak <l.lunak@kde.org>

 Distributed under the terms of the GNU General Public License version 2.
 
 $Id: actions_listview_widget.cpp,v 1.13 2001/07/06 18:45:20 seli Exp $

****************************************************************************/

#define __ACTIONS_LISTVIEW_WIDGET_CPP

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "actions_listview_widget.h"

#include <qheader.h>

#include <klistview.h>
#include <klocale.h>
#include <kdebug.h>

#include <actions.h>
#include <action_data.h>
#include <triggers.h>

#include "kcmkhotkeys.h"

namespace KHotKeys
{

Actions_listview_widget::Actions_listview_widget( QWidget* parent_P, const char* name_P )
    : Actions_listview_widget_ui( parent_P, name_P ), recent_item( NULL ),
        saved_current_item( NULL )
    {
    actions_listview->setSorting( 0 );
    actions_listview->header()->hide();
    actions_listview->setRootIsDecorated( true ); // CHECKME
    connect( actions_listview, SIGNAL( clicked( QListViewItem*, const QPoint&, int )),
        SLOT( clicked( QListViewItem*, const QPoint&, int )));
    connect( actions_listview, SIGNAL( currentChanged( QListViewItem* )),
        SLOT( current_changed( QListViewItem* )));
    // KHotKeys::Module::changed()
    }
    
void Actions_listview_widget::action_name_changed( const QString& )
    {
    actions_listview->repaintItem( current_action());
    }

void Actions_listview_widget::set_action_data( Action_data_base* data_P, bool recent_action_P )
    {
    if( recent_action_P )
        {
        assert( recent_item != NULL );
        recent_item->set_data( data_P );
        }
    else
        saved_current_item->set_data( data_P );
    }

// this makes it possible to unselect selected item by clicking on an empty space in the listview    
void Actions_listview_widget::clicked( QListViewItem* item_P, const QPoint& point_P, int )
    {
    // CHECKME item_P may be NULL if user clicks on the expansion + , so detect this too
    item_P = actions_listview->itemAt( actions_listview->viewport()->mapFromGlobal( point_P ));
    if( item_P == NULL )
        {
        if( saved_current_item != NULL )
            current_changed( NULL );
        actions_listview->clearSelection();
        actions_listview->clearFocus();
        return;
        }
    if( saved_current_item == NULL && item_P == actions_listview->currentItem())
        current_changed( item_P ); // HACK
    }
    
void Actions_listview_widget::current_changed( QListViewItem* item_P )
    {
    if( item_P == saved_current_item )
        return;
    recent_item = saved_current_item;
    saved_current_item = static_cast< Action_listview_item* >( item_P );
    actions_listview->setSelected( item_P, true );
    emit current_action_changed();
    }
    
void Actions_listview_widget::new_action( Action_data_base* data_P )
    {
    Action_listview_item* parent = NULL;
    if( current_action() != NULL )
        {
        if( dynamic_cast< Action_data_group* >( current_action_data()) != NULL ) // is a group
            parent = current_action();
        else if( current_action()->parent() != NULL )
            parent = static_cast< Action_listview_item* >( current_action()->parent());
        }
    Action_listview_item* tmp;
    if( parent )
        {
        tmp = new Action_listview_item( parent, data_P );
        parent->setOpen( true );
        }
    else
        tmp = new Action_listview_item( actions_listview, data_P );
    recent_item = saved_current_item;
    saved_current_item = tmp;
    actions_listview->setSelected( tmp, true );
    }

void Actions_listview_widget::delete_action()
    {
    while( QListViewItem* child = current_action()->firstChild())
        delete child;
    QListViewItem* nw = current_action()->itemAbove();
    if( nw == NULL )
        nw = current_action()->itemBelow();
    delete saved_current_item;
    recent_item = NULL;
    if( nw != NULL )
        {
        saved_current_item = static_cast< Action_listview_item* >( nw );
        actions_listview->setSelected( nw, true );
        }
    else
        saved_current_item = NULL;
    }

void Actions_listview_widget::build_up()
    {
    build_up_recursively( module->actions_root(), NULL );
    }
    
void Actions_listview_widget::build_up_recursively( Action_data_group* parent_P,
    Action_listview_item* item_parent_P )
    {
    Action_listview_item* prev = NULL;
    for( Action_data_group::Iterator it = parent_P->first_child();
         it;
         ++it )
        {
        if( prev == NULL )
            {
            if( item_parent_P == NULL )
                prev = new Action_listview_item( actions_listview, ( *it )); 
            else
                prev = new Action_listview_item( item_parent_P, ( *it )); 
            }
        else
            {
            if( item_parent_P == NULL )
                prev = new Action_listview_item( actions_listview, prev, ( *it )); 
            else
                prev = new Action_listview_item( item_parent_P, prev, ( *it )); 
            }
        Action_data_group* grp = dynamic_cast< Action_data_group* >( *it );
        if( grp != NULL )
            build_up_recursively( grp, prev );
        }
    }
    
// Action_listview_item

QString Action_listview_item::text( int column_P ) const
    {
    return column_P == 0 ? data()->name() : QString::null;
    }

} // namespace KHotKeys

#include "actions_listview_widget.moc"
