/****************************************************************************

 KHotKeys
 
 Copyright (C) 1999-2001 Lubos Lunak <l.lunak@kde.org>

 Distributed under the terms of the GNU General Public License version 2.
 
 $Id: condition_list_widget.cpp,v 1.3 2001/07/15 14:08:29 seli Exp $

****************************************************************************/

#define __CONDITIONS_LIST_WIDGET_CPP

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "condition_list_widget.h"

#include <assert.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qlineedit.h>
#include <qpopupmenu.h>

#include <klistview.h>
#include <kdebug.h>
#include <klocale.h>

#include <conditions.h>

#include "windowdef_list_widget.h"

#include "kcmkhotkeys.h"

namespace KHotKeys
{

// Condition_list_widget

Condition_list_widget::Condition_list_widget( QWidget* parent_P, const char* name_P )
    : Condition_list_widget_ui( parent_P, name_P ), selected_item( NULL )
    {
    QPopupMenu* popup = new QPopupMenu; // CHECKME looks like setting parent doesn't work
    popup->insertItem( i18n( "Active window" ), TYPE_ACTIVE_WINDOW );
    popup->insertItem( i18n( "Existing window" ), TYPE_EXISTING_WINDOW );
    popup->insertItem( i18n( "Not" ), TYPE_NOT );
    connect( popup, SIGNAL( activated( int )), SLOT( new_selected( int )));
    new_button->setPopup( popup );
    conditions_listview->header()->hide();
    conditions_listview->setSorting( -1 );
    conditions_listview->setRootIsDecorated( true ); // CHECKME
    copy_button->setEnabled( false );    
    modify_button->setEnabled( false );    
    delete_button->setEnabled( false );    
    clear_data();
    // KHotKeys::Module::changed()
    connect( new_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( copy_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( modify_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( delete_button, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( comment_lineedit, SIGNAL( textChanged( const QString& )),
        module, SLOT( changed()));
    }
    
Condition_list_widget::~Condition_list_widget()
    {
    delete new_button->popup();
    }

void Condition_list_widget::clear_data()
    {
    comment_lineedit->setText( "" );
    conditions_listview->clear();
    }
    
void Condition_list_widget::set_data( const Condition_list* data_P )
    {
    comment_lineedit->setText( data_P->comment());
    conditions_listview->clear();
    insert_listview_items( data_P, conditions_listview, NULL );
    }
    
void Condition_list_widget::insert_listview_items( const Condition_list_base* parent_P,
    QListView* parent1_P, QListViewItem* parent2_P )
    {
    Condition_list_item* prev = NULL;
    for( Condition_list_base::Iterator it( *parent_P );
         *it;
         ++it )
        {
        prev = create_listview_item( *it, parent1_P, parent2_P, prev, true );
        if( Condition_list_base* group = dynamic_cast< Condition_list_base* >( *it ))
            insert_listview_items( group, NULL, prev );
        }
    }
    
Condition_list* Condition_list_widget::get_data( Action_data_base* data_P ) const
    {
// CHECKME TODO hmm, tady to bude chtit asi i children :(
    Condition_list* list = new Condition_list( comment_lineedit->text(), data_P );
    get_listview_items( list, conditions_listview->firstChild());
    return list;
    }

void Condition_list_widget::get_listview_items( Condition_list_base* list_P,
    QListViewItem* first_item_P ) const
    {
    list_P->clear();
    for( QListViewItem* pos = first_item_P;
         pos != NULL;
         pos = pos->nextSibling())
        {
        Condition* cond = static_cast< Condition_list_item* >( pos )->condition()->copy( list_P );
        list_P->append( cond );
        if( Condition_list_base* group = dynamic_cast< Condition_list_base* >( cond ))
            get_listview_items( group, pos->firstChild());
        }
    }

void Condition_list_widget::new_selected( int type_P )
    {
    Condition_dialog* dlg = NULL;
    Condition* condition = NULL;
    switch( type_P )
        {
        case TYPE_ACTIVE_WINDOW: // Active_window_condition
            dlg = new Active_window_condition_dialog(
                new Active_window_condition( new Windowdef_list( "" ), NULL )); // CHECKME NULL
          break;
        case TYPE_EXISTING_WINDOW: // Existing_window_condition
            dlg = new Existing_window_condition_dialog(
                new Existing_window_condition( new Windowdef_list( "" ), NULL )); // CHECKME NULL
          break;
        case TYPE_NOT: // Not_condition
            condition = new Not_condition( NULL, NULL );
          break;
        }
    if( dlg != NULL )
        {
        condition = dlg->edit_condition();
        delete dlg;
        }
    if( condition != NULL )
        {
        if( selected_item
            && dynamic_cast< Condition_list_base* >( selected_item->condition()) != NULL )
            {
            conditions_listview->setSelected( create_listview_item( condition,
                NULL, selected_item, NULL, true ), true );
            }
        else
            conditions_listview->setSelected( create_listview_item( condition,
                conditions_listview, NULL, selected_item, true ), true );
        }
    }
    
void Condition_list_widget::copy_pressed()
    {
    conditions_listview->setSelected( create_listview_item( selected_item->condition(),
        selected_item->parent() ? NULL : conditions_listview, selected_item->parent(),
        selected_item, true ), true );
    }
    
void Condition_list_widget::delete_pressed()
    {
    delete selected_item; // CHECKME snad vyvola signaly pro enable()
    }
    
void Condition_list_widget::modify_pressed()
    {
    edit_listview_item( selected_item );
    }
    
void Condition_list_widget::current_changed( QListViewItem* item_P )
    {
    if( item_P == selected_item )
        return;
    selected_item = static_cast< Condition_list_item* >( item_P );
    conditions_listview->setSelected( item_P, true );
    copy_button->setEnabled( item_P != NULL );
    modify_button->setEnabled( item_P != NULL );
    delete_button->setEnabled( item_P != NULL );
    }

Condition_list_item* Condition_list_widget::create_listview_item( Condition* condition_P,
    QListView* parent1_P, QListViewItem* parent2_P, QListViewItem* after_P, bool copy_P )
    {
    Condition* new_win = copy_P ? condition_P->copy( NULL ) : condition_P;
    if( after_P == NULL )
        {
        if( parent1_P == NULL )
            return new Condition_list_item( parent2_P, new_win );
        else
            return new Condition_list_item( parent1_P, new_win );
        }
    else
        {
        if( parent1_P == NULL )
            return new Condition_list_item( parent2_P, after_P, new_win );
        else
            return new Condition_list_item( parent1_P, after_P, new_win );
        }
    }

void Condition_list_widget::edit_listview_item( Condition_list_item* item_P )
    {
    Condition_dialog* dlg = NULL;
    if( Active_window_condition* condition
        = dynamic_cast< Active_window_condition* >( item_P->condition()))
        dlg = new Active_window_condition_dialog( condition );
    else if( Existing_window_condition* condition
        = dynamic_cast< Existing_window_condition* >( item_P->condition()))
        dlg = new Existing_window_condition_dialog( condition );
    else // CHECKME TODO pridat dalsi
        assert( false );
    Condition* new_condition = dlg->edit_condition();
    if( new_condition != NULL )
        item_P->set_condition( new_condition );
    delete dlg;
    item_P->widthChanged();
    }

// Condition_list_item

QString Condition_list_item::text( int column_P ) const
    {
    return column_P == 0 ? condition()->description() : QString::null;
    }
    
Condition_list_item::~Condition_list_item()
    {
    delete _condition;
    }

// Active_window_condition_dialog

Active_window_condition_dialog::Active_window_condition_dialog( 
    Active_window_condition* condition_P )
    : KDialogBase( NULL, NULL, true, i18n( "Window details" ), Ok | Cancel ), condition( NULL )
    {
    widget = new Windowdef_list_widget( this );
    widget->set_data( condition_P->window());
    setMainWidget( widget );
    }
    
Condition* Active_window_condition_dialog::edit_condition()
    {
    exec();
    return condition;
    }
    
void Active_window_condition_dialog::accept()
    {
    KDialogBase::accept();
    condition = new Active_window_condition( widget->get_data(), NULL ); // CHECKME NULL ?
    }
    
// Existing_window_condition_dialog

Existing_window_condition_dialog::Existing_window_condition_dialog( 
    Existing_window_condition* condition_P )
    : KDialogBase( NULL, NULL, true, i18n( "Window details" ), Ok | Cancel ), condition( NULL )
    {
    widget = new Windowdef_list_widget( this );
    widget->set_data( condition_P->window());
    setMainWidget( widget );
    }
    
Condition* Existing_window_condition_dialog::edit_condition()
    {
    exec();
    return condition;
    }
    
void Existing_window_condition_dialog::accept()
    {
    KDialogBase::accept();
    condition = new Existing_window_condition( widget->get_data(), NULL ); // CHECKME NULL ?
    }
    
} // namespace KHotKeys

#include "condition_list_widget.moc"
