#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <klocale.h>
#include <kmessagebox.h>

#include "gesturerecordpage.h"
#include "gesturerecorder.h"
#include "gesturedrawer.h"
#include "gesture.h"

namespace KHotKeys {

GestureRecordPage::GestureRecordPage(const QString &gesture,
                                     QWidget *parent, const char *name)
  : QVBox(parent, name),
    _recorder(NULL), _resetButton(NULL),
    _tryOne(NULL), _tryTwo(NULL), _tryThree(NULL), _gest(QString::null),
    _tryCount(1)
  {
  QString message;

  message += i18n("Draw the gesture you would like to record below. Press ");
  message += i18n("and hold the left mouse button while drawing, and release ");
  message += i18n("when your done.\n\n");

  message += i18n("You will be required to draw the gesture 3 times. After ");
  message += i18n("each drawing, if they match, the indicators below will ");
  message += i18n("change to represent which step you are on.\n\n");

  message += i18n("If at any point they do not match, you will be required to ");
  message += i18n("restart. If you want to force a restart, use the reset ");
  message += i18n("button below.\n\nDraw here:");

  QLabel *label = new QLabel(message, this, "label");
  label->setAlignment(QLabel::AlignLeft | QLabel::WordBreak |
                      QLabel::AlignVCenter);
  
  _recorder = new GestureRecorder(this, "recorder");
  _recorder->setMinimumHeight(150);
  setStretchFactor(_recorder, 1);
  connect(_recorder, SIGNAL(recorded(const QString &)),
	  this, SLOT(slotRecorded(const QString &)));

  QHBox *hBox = new QHBox(this, "hbox");

  _tryOne = new GestureDrawer(hBox, "tryOne");
  _tryOne->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  _tryOne->setMinimumSize(30, 20);
  
  _tryTwo = new GestureDrawer(hBox, "tryTwo");
  _tryTwo->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  _tryTwo->setMinimumSize(30, 20);
 
  _tryThree = new GestureDrawer(hBox, "tryThree");
  _tryThree->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  _tryThree->setMinimumSize(30, 20);
 
  QWidget *spacer = new QWidget(hBox, "spacer");
  hBox->setStretchFactor(spacer, 1);

  _resetButton = new QPushButton(i18n("&Reset"), hBox, "resetButton");
  connect(_resetButton, SIGNAL(clicked()),
	  this, SLOT(slotResetClicked()));
      
      
   
  // initialize
  if (gesture != QString::null)
      {
      slotRecorded(gesture);
      slotRecorded(gesture);
      slotRecorded(gesture);
      }
  }

GestureRecordPage::~GestureRecordPage()
  {
  }

void GestureRecordPage::slotRecorded(const QString &data)
  {
  switch (_tryCount)
    {
    case 1:
      {
	_gest = data;
	_tryOne->setData(_gest);
	_tryCount++;
	}
	break;
    
    case 2:
     {
     if (_gest == data)
       {
       _tryTwo->setData(data);
       _tryCount++;
       }
     else
       {
       KMessageBox::sorry(this, i18n("Sorry, your gestures didn't match."));
       slotResetClicked();
       }
     break;
     }
     
    case 3:
    {
     if (_gest == data)
       {
       _tryThree->setData(data);
       _tryCount++;
       emit readyForNext(true);
       }
     else
       {
       KMessageBox::sorry(this, i18n("Sorry, your gestures didn't match."));
       slotResetClicked();
       }
     break;
     }
    default:
	KMessageBox::information(this, i18n("You have already completed the three required drawings. Either press Ok to save or Reset to try again."));
    }
  }

void GestureRecordPage::slotResetClicked()
  {
  _gest = QString::null;
  
  _tryOne->setData(QString::null);
  _tryTwo->setData(QString::null);
  _tryThree->setData(QString::null);

  _tryCount = 1;

  emit readyForNext(false);
  }
  
} // end namespace
