/****************************************************************************

 KHotKeys
 
 Copyright (C) 1999-2001 Lubos Lunak <l.lunak@kde.org>

 Distributed under the terms of the GNU General Public License version 2.
 
 $Id: windowdef_simple_widget.cpp,v 1.12 2001/07/07 11:58:05 seli Exp $

****************************************************************************/

#define __WINDOWDEF_SIMPLE_WIDGET_CPP

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "windowdef_simple_widget.h"

#include <qlineedit.h>
#include <qcombobox.h>
#include <qcheckbox.h>

#include <windows.h>

#include "windowdef_list_widget.h"
#include "kcmkhotkeys.h"

namespace KHotKeys
{

Windowdef_simple_widget::Windowdef_simple_widget( QWidget* parent_P, const char* name_P )
    : Windowdef_simple_widget_ui( parent_P, name_P )
    {
    window_title_lineedit->setEnabled( false );
    window_class_lineedit->setEnabled( false );
    window_command_lineedit->setEnabled( false );
    connect( autodetect_button, SIGNAL( clicked()), SIGNAL( autodetect_signal()));
    autodetect_button->setEnabled( false );
    clear_data();
    // KHotKeys::Module::changed()
    connect( window_title_combo, SIGNAL( activated( int )),
        module, SLOT( changed()));
    connect( window_title_lineedit, SIGNAL( textChanged( const QString& )),
        module, SLOT( changed()));
    connect( window_class_combo, SIGNAL( activated( int )),
        module, SLOT( changed()));
    connect( window_class_lineedit, SIGNAL( textChanged( const QString& )),
        module, SLOT( changed()));
    connect( window_command_combo, SIGNAL( activated( int )),
        module, SLOT( changed()));
    connect( window_command_lineedit, SIGNAL( textChanged( const QString& )),
        module, SLOT( changed()));
    connect( type_normal_checkbox, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( type_dialog_checkbox, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( type_dock_checkbox, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( type_desktop_checkbox, SIGNAL( clicked()),
        module, SLOT( changed()));
    connect( comment_lineedit, SIGNAL( textChanged( const QString& )),
        module, SLOT( changed()));
    }

void Windowdef_simple_widget::clear_data()
    {
    comment_lineedit->setText( "" );
    window_title_combo->setCurrentItem( 0 );
    window_title_lineedit->setText( "" );    
    window_title_lineedit->setEnabled( false );
    window_class_combo->setCurrentItem( 0 );
    window_class_lineedit->setText( "" );
    window_class_lineedit->setEnabled( false );
    window_command_combo->setCurrentItem( 0 );
    window_command_lineedit->setText( "" );
    window_command_lineedit->setEnabled( false );
    type_normal_checkbox->setChecked( true );
    type_dialog_checkbox->setChecked( true );
    type_dock_checkbox->setChecked( false );
//    type_tool_checkbox->setChecked( false );
//    type_menu_checkbox->setChecked( false );
    type_desktop_checkbox->setChecked( false );
    }
    
void Windowdef_simple_widget::set_data( const Windowdef_simple* data_P )
    {
    comment_lineedit->setText( data_P->comment());
    window_title_combo->setCurrentItem( data_P->title_match_type());
    window_title_lineedit->setText( data_P->title());    
    window_title_lineedit->setEnabled( window_title_combo->currentItem() != 0 );
    window_class_combo->setCurrentItem( data_P->wclass_match_type());
    window_class_lineedit->setText( data_P->wclass());
    window_class_lineedit->setEnabled( window_class_combo->currentItem() != 0 );
    window_command_combo->setCurrentItem( data_P->command_match_type());
    window_command_lineedit->setText( data_P->command());
    window_command_lineedit->setEnabled( window_command_combo->currentItem() != 0 );
    type_normal_checkbox->setChecked( data_P->type_match( Windowdef_simple::WINDOW_TYPE_NORMAL ));
    type_dialog_checkbox->setChecked( data_P->type_match( Windowdef_simple::WINDOW_TYPE_DIALOG ));
    type_dock_checkbox->setChecked( data_P->type_match( Windowdef_simple::WINDOW_TYPE_DOCK ));
//    type_tool_checkbox->setChecked( data_P->type_match( Windowdef_simple::WINDOW_TYPE_TOOL ));
//    type_menu_checkbox->setChecked( data_P->type_match( Windowdef_simple::WINDOW_TYPE_MENU ));
    type_desktop_checkbox->setChecked( 
        data_P->type_match( Windowdef_simple::WINDOW_TYPE_DESKTOP ));
    }
    
Windowdef_simple* Windowdef_simple_widget::get_data() const
    {
    return new Windowdef_simple( comment_lineedit->text(), window_title_lineedit->text(),
        static_cast< Windowdef_simple::substr_type_t >( window_title_combo->currentItem()),
        window_class_lineedit->text(),
        static_cast< Windowdef_simple::substr_type_t >( window_class_combo->currentItem()),
        window_command_lineedit->text(),
        static_cast< Windowdef_simple::substr_type_t >( window_command_combo->currentItem()),
            ( type_normal_checkbox->isChecked() ? Windowdef_simple::WINDOW_TYPE_NORMAL : 0 )
            | ( type_dialog_checkbox->isChecked() ? Windowdef_simple::WINDOW_TYPE_DIALOG : 0 )
            | ( type_dock_checkbox->isChecked() ? Windowdef_simple::WINDOW_TYPE_DOCK : 0 )
//            | ( type_tool_checkbox->isChecked() ? Windowdef_simple::WINDOW_TYPE_TOOL : 0 )
//            | ( type_menu_checkbox->isChecked() ? Windowdef_simple::WINDOW_TYPE_MENU : 0 )
            | ( type_desktop_checkbox->isChecked() ? Windowdef_simple::WINDOW_TYPE_DESKTOP : 0 )
        );
    }
    
void Windowdef_simple_widget::window_command_combo_changed( int item_P )
    { // CHECKME nebo je connect() na highlighted() ?
    window_command_lineedit->setEnabled( item_P != 0 );
    }
    
void Windowdef_simple_widget::window_class_combo_changed( int item_P )
    {
    window_class_lineedit->setEnabled( item_P != 0 );
    }
    
void Windowdef_simple_widget::window_title_combo_changed( int item_P )
    {
    window_title_lineedit->setEnabled( item_P != 0 );
    }
    
void Windowdef_simple_widget::set_autodetect( QObject* obj_P, const char* slot_P )
    {
    disconnect( SIGNAL( autodetect_signal()));
    if( obj_P != NULL )
        connect( this, SIGNAL( autodetect_signal()), obj_P, slot_P );
    autodetect_button->setEnabled( obj_P != NULL );
    }
    
void Windowdef_simple_widget::autodetect_clicked()
    {
    emit autodetect_signal();
    autodetect();
    }
    
void Windowdef_simple_widget::autodetect()
    {
    // CHECKME TODO
    }
    
} // namespace KHotKeys

#include "windowdef_simple_widget.moc"
